@@ -28,26 +28,28 @@ from abc import ABC, abstractmethod
 from .cryptography import CryptographyKey
 
 
+# noinspection PyAbstractClass
 class AsymmetricKey(CryptographyKey, ABC):
 
-    RSA = 'RSA'
-    ECC = 'ECC'
+    # RSA = 'RSA'  # -- "RSA/ECB/PKCS1Padding", "SHA256withRSA"
+    # ECC = 'ECC'
+    pass
 
 
-class SignKey(AsymmetricKey):
+class SignKey(AsymmetricKey, ABC):
 
     @abstractmethod
     def sign(self, data: bytes) -> bytes:
         """
-        signature = sign(data, SK);
+        signature = sign(data, SK)
 
         :param data: message data
-        :return:     signature
+        :return: signature
         """
         raise NotImplemented
 
 
-class VerifyKey(AsymmetricKey):
+class VerifyKey(AsymmetricKey, ABC):
 
     @abstractmethod
     def verify(self, data: bytes, signature: bytes) -> bool:
@@ -56,24 +58,17 @@ class VerifyKey(AsymmetricKey):
 
         :param data:      message data
         :param signature: signature of message data
-        :return:          True on signature matched
+        :return: True on signature matched
         """
         raise NotImplemented
 
-    def match(self, key: SignKey):
+    @abstractmethod
+    def match_sign_key(self, key: SignKey) -> bool:
         """
-        OK = verify(data, sign(data, SK), PK)
+        signature = sign(data, SK)
+        OK = verify(data, signature, PK)
 
         :param key: private key
-        :return:    True on signature matched
+        :return: True on signature matched
         """
-        return asymmetric_keys_match(sign_key=key, verify_key=self)
-
-
-promise = 'Moky loves May Lee forever!'.encode('utf-8')
-
-
-def asymmetric_keys_match(sign_key: SignKey, verify_key: VerifyKey) -> bool:
-    # try to verify with signature
-    signature = sign_key.sign(data=promise)
-    return verify_key.verify(data=promise, signature=signature)
+        raise NotImplemented
