@@ -23,13 +23,13 @@
 # SOFTWARE.
 # ==============================================================================
 
-from abc import abstractmethod
-from typing import Optional
+from abc import ABC, abstractmethod
+from typing import Optional, Dict
 
-from .dictionary import Map
+from ..types import Mapper
 
 
-class CryptographyKey(Map):
+class CryptographyKey(Mapper, ABC):
     """Cryptography key with designated algorithm
 
         Cryptography Key
@@ -43,6 +43,7 @@ class CryptographyKey(Map):
     """
 
     @property
+    @abstractmethod
     def algorithm(self) -> str:
         """
         Get key algorithm name
@@ -62,49 +63,46 @@ class CryptographyKey(Map):
         raise NotImplemented
 
 
-class EncryptKey(CryptographyKey):
+class EncryptKey(CryptographyKey, ABC):
 
     @abstractmethod
-    def encrypt(self, data: bytes) -> bytes:
+    def encrypt(self, data: bytes, extra: Optional[Dict] = None) -> bytes:
         """
-        ciphertext = encrypt(plaintext, PW)
-        ciphertext = encrypt(plaintext, PK)
+        1. Symmetric Key:
+            ciphertext = encrypt(plaintext, PW)
+        2. Asymmetric Public Key:
+            ciphertext = encrypt(plaintext, PK)
 
-        :param data: plaintext
-        :return:     ciphertext
+        :param data:  plaintext
+        :param extra: store extra variables ('IV' for 'AES')
+        :return: ciphertext
         """
         raise NotImplemented
 
 
-class DecryptKey(CryptographyKey):
+class DecryptKey(CryptographyKey, ABC):
 
     @abstractmethod
-    def decrypt(self, data: bytes) -> Optional[bytes]:
+    def decrypt(self, data: bytes, params: Optional[Dict] = None) -> Optional[bytes]:
         """
-        plaintext = decrypt(ciphertext, PW);
-        plaintext = decrypt(ciphertext, SK);
+        1. Symmetric Key:
+            plaintext = decrypt(ciphertext, PW)
+        2. Asymmetric Private Key:
+            plaintext = decrypt(ciphertext, SK)
 
-        :param data: ciphertext
-        :return:     plaintext
+        :param data:   ciphertext
+        :param params: extra params ('IV' for 'AES')
+        :return: plaintext
         """
         raise NotImplemented
 
-    def match(self, key: EncryptKey):
+    @abstractmethod
+    def match_encrypt_key(self, key: EncryptKey) -> bool:
         """
-        OK = decrypt(encrypt(data, SK), PK) == data
+        CT = encrypt(data, PK)
+        OK = decrypt(CT, SK) == data
 
-        :param key: encrypt key
-        :return:    False on error
+        :param key: encrypt (public) key
+        :return: False on error
         """
-        return keys_match(encrypt_key=key, decrypt_key=self)
-
-
-def key_algorithm(key: dict) -> str:
-    return key.get('algorithm')
-
-
-promise = 'Moky loves May Lee forever!'.encode('utf-8')
-
-
-def keys_match(encrypt_key: EncryptKey, decrypt_key: DecryptKey) -> bool:
-    return decrypt_key.decrypt(encrypt_key.encrypt(promise)) == promise
+        raise NotImplemented
